import re

from api.objects.type import (IntType, LongType,
                              SimpleType, ObjectIdType,
                              StringType, SimpleStringType, UniqueStringType, MappedStringType, SemesterStringType,
                              DatetimeType)

TYPE_INT = IntType()
TYPE_LONG = LongType()
TYPE_BOOLEAN = SimpleType("boolean", lambda db: bool(db), lambda json_value: json_value.as_bool())
TYPE_STRING_SHORT = StringType(256)
TYPE_STRING_LONG = StringType(8192)
TYPE_URL = SimpleStringType("url", 1024, 0, None)

# Also update in API specification when modifying!
FORBIDDEN_COURSE_ID_STRING_VALUES = ["courses", "faq", "imprint", "licenses", "search", "site", "internal"]
# noinspection RegExpUnnecessaryNonCapturingGroup
TYPE_COURSE_ID_STRING = UniqueStringType(
    "course_id_string",
    100,
    1,
    re.compile(f"(?!(?:{'|'.join(FORBIDDEN_COURSE_ID_STRING_VALUES)})$)[a-zA-Z0-9_-]+"),
    "courses_data",
    "id",
    "handle"
)
TYPE_SEMESTER_STRING = SemesterStringType()
TYPE_DURATION = IntType(0, 2147483647)
TYPE_DATETIME = DatetimeType()
TYPE_TIMESTAMP = IntType(0, 2147483647)
TYPE_ANNOUNCEMENT_TYPE = MappedStringType("announcement_type", False, {
        "info": "info",
        "warning": "warning",
        "important": "important"
    })  # Currently not in db reflected
TYPE_ANNOUNCEMENT_VISIBILITY = MappedStringType("announcement_visibility", False, {
        "only_main_page": "only_main_page",
        "all_pages": "all_pages",
        "all_pages_and_embed": "all_pages_and_embed"
    })  # Currently not in db reflected
TYPE_LEGACY_ANNOUNCEMENT_TYPE = MappedStringType("legacy_announcement_type", False, {
        "info_main_page": 0,
        "info_all_pages": 1,
        "warning_all_pages": 2,
        "important_all_pages_embed": 3
    })
TYPE_FEATURED_TYPE = MappedStringType("featured_type", False, {
        "plain": "plain",
        "image": "image",
        "lecture": "video",
        "course_list": "courses"
    })
TYPE_COURSE_FILTER_CRITERIA = MappedStringType("course_filter_criteria", False, {
        "title": "title",
        "semester": "semester",
        "organizer": "organizer",
        "topic": "subject"
    })
TYPE_PERMISSION_TYPE = MappedStringType("permission_type", False, {
        "public": "public",
        "rwth": "rwth",
        "moodle": "moodle",
        "fsmpi": "fsmpi",
        "password": "password",
        "none": "none"
    })
TYPE_USER_ID = ObjectIdType(
    "user_id",
    False,
    "users",
    "id",
    False,
    "The user does not exist")
TYPE_LECTURE_ID = ObjectIdType(
    "lecture_id",
    False,
    "lectures_data",
    "id",
    True,
    "The lecture does not exist")
