from api.database import *
from api.miscellaneous import *
from api.settings import Settings, SettingsSection, SettingsSubSectionEntry, SettingsBooleanEntry


def user_info_list_db_to_json(user_list_db: list[DbResultRow]):
    return list(map(user_info_db_to_json, user_list_db))


def user_info_db_to_json(user_db: DbResultRow):
    return {
        "id": user_db["id"],
        "id_string": user_db["name"],
        "name": user_db["realname"]
    }


_SETTINGS = Settings([
    SettingsSection("notifications", [
        SettingsBooleanEntry("notifications_enabled", "mail_notifications"),
        SettingsSubSectionEntry("notifications_responsible_courses", [
            SettingsBooleanEntry("notifications_responsible_courses_new_media_source", "notify_new_video"),
            SettingsBooleanEntry("notifications_responsible_courses_chapter_submitted", "notify_chapter_submitted"),
            SettingsBooleanEntry("notifications_responsible_courses_changes", "notify_edit"),
        ])
    ])
])


_SQL_GET_USER = PreparedStatement(f"""
SELECT *
FROM "users"
WHERE "id" = ?
""")


def get_user_settings(user_id: int) -> dict or None:
    user_list_db = db_pool.execute_read_statement_in_transaction(_SQL_GET_USER, user_id)
    if len(user_list_db) == 0:
        return None
    user_db = user_list_db[0]
    return _SETTINGS.to_json(user_db)


def update_user_settings(user_id: int, updates_json: CJsonObject):
    updates_db: list[tuple[str, str]] = []
    for entry_id in updates_json.keys():
        entry = _SETTINGS.get_value_entry(entry_id)
        if entry is None:
            raise ApiClientException(ERROR_BAD_REQUEST(f"Unknown user settings id {truncate_string(entry_id)}"))
        updates_db.append((
            entry.db_column,
            entry.json_value_to_db(updates_json.get(entry_id))
        ))
    if len(updates_db) == 0:
        return
    db_pool.execute_write_transaction(lambda trans: trans.execute_statement_and_commit(
        f"""
        UPDATE "users"
        SET {",".join(map(lambda tup: f'"{tup[0]}" = ?', updates_db))}
        WHERE "id" = ?
        """, *map(lambda tup: tup[1], updates_db), user_id)
    )
