import unittest

from api.miscellaneous.constants import *

from api_test import ApiTest

# The python json parser does not unescape the escaped strings, so we need to put the extra \ here.
_TEST_DATA_JOB_1 = {
    "id": 1,
    "type": "thumbnail",
    "priority": 0,
    "state": "ready",
    "is_canceled": False,
    "creation_time": "2024-03-18T18:53:02",
    "data": "{\"src\": \"pub/07ws-diskrete/07ws-diskrete-071211.mp4\", \"filename\": \"l_25.jpg\"}",
    "status": "{}"
}
_TEST_DATA_JOB_2 = {
    "id": 2,
    "type": "thumbnail",
    "priority": 0,
    "state": "deleted",
    "is_canceled": False,
    "creation_time": "2024-03-18T18:52:00",
    "data": "{\"src\": \"pub/07ws-diskrete/07ws-diskrete-071211.mp4\", \"filename\": \"l_25.jpg\"}",
    "status": "{}"
}
_TEST_DATA_JOB_3 = {
    "id": 3,
    "type": "remux",
    "priority": 0,
    "last_worker_ping": "2024-03-19T12:38:38",
    "state": "running",
    "is_canceled": False,
    "creation_time": "2024-03-17T09:20:02",
    "data": "{\"src\": \"pub/07ws-diskrete/07ws-diskrete-071211.mp4\"}",
    "status": "{}",
    "worker_id": "work1",
    "start_time": "2024-03-19T12:37:38"
}
_TEST_DATA_JOB_4 = {
    "id": 4,
    "type": "thumbnail",
    "priority": 0,
    "last_worker_ping": "2024-03-19T10:30:41",
    "state": "finished",
    "is_canceled": False,
    "creation_time": "2024-03-17T09:21:02",
    "data": "{\"src\": \"pub/07ws-diskrete/07ws-diskrete-071211.mp4\", \"filename\": \"l_25.jpg\"}",
    "status": "{}",
    "worker_id": "work2",
    "start_time": "2024-03-19T10:29:10",
    "finish_time": "2024-03-19T10:30:41"
}
_TEST_DATA_JOB_5 = {
    "id": 5,
    "type": "thumbnail",
    "priority": 0,
    "last_worker_ping": "2024-03-19T10:30:41",
    "state": "failed",
    "is_canceled": False,
    "creation_time": "2024-03-17T09:21:01",
    "data": "{\"src\": \"pub/07ws-diskrete/07ws-diskrete-071211.mp4\", \"filename\": \"l_25.jpg\"}",
    "status": "{}",
    "worker_id": "work2",
    "start_time": "2024-03-19T10:29:10"
}

_TEST_DATA_WORKER_1 = {
    "hostname": "work1",
    "last_ping": "2024-03-26T08:12:49"
}


class JobTest(ApiTest):
    
    def test_get_jobs(self):
        self.do_json_request(
            "GET",
            "/jobs",
            expected_response_code=HTTP_401_UNAUTHORIZED
        )
        self.do_json_request(
            "GET",
            "/jobs",
            None,
            {
                "page_count": 1,
                "page": [
                    _TEST_DATA_JOB_1,
                    _TEST_DATA_JOB_2,
                    _TEST_DATA_JOB_4,
                    _TEST_DATA_JOB_5,
                    _TEST_DATA_JOB_3,
                ],
                "workers": [_TEST_DATA_WORKER_1]
            },
            use_moderator_login=True
        )
        self.do_json_request(
            "GET",
            "/jobs?entries_per_page=10&page=0",
            None,
            {
                "page_count": 1,
                "page": [
                    _TEST_DATA_JOB_1,
                    _TEST_DATA_JOB_2,
                    _TEST_DATA_JOB_4,
                    _TEST_DATA_JOB_5,
                    _TEST_DATA_JOB_3,
                ],
                "workers": [_TEST_DATA_WORKER_1]
            },
            use_moderator_login=True
        )
        self.do_json_request(
            "GET",
            "/jobs?entries_per_page=10&page=1",
            None,
            {
                "page_count": 1,
                "page": [],
                "workers": [_TEST_DATA_WORKER_1]
            },
            use_moderator_login=True
        )
        self.do_json_request(
            "GET",
            "/jobs?entries_per_page=10&page=0&state=ready",
            None,
            {
                "page_count": 1,
                "page": [
                    _TEST_DATA_JOB_1
                ],
                "workers": [_TEST_DATA_WORKER_1]
            },
            use_moderator_login=True
        )
        self.do_json_request(
            "GET",
            "/jobs?entries_per_page=10&page=0&state=running",
            None,
            {
                "page_count": 1,
                "page": [
                    _TEST_DATA_JOB_3
                ],
                "workers": [_TEST_DATA_WORKER_1]
            },
            use_moderator_login=True
        )
        self.do_json_request(
            "GET",
            "/jobs?entries_per_page=10&page=0&type=thumbnail",
            None,
            {
                "page_count": 1,
                "page": [
                    _TEST_DATA_JOB_1,
                    _TEST_DATA_JOB_2,
                    _TEST_DATA_JOB_4,
                    _TEST_DATA_JOB_5,
                ],
                "workers": [_TEST_DATA_WORKER_1]
            },
            use_moderator_login=True
        )
        self.do_json_request(
            "GET",
            "/jobs?entries_per_page=10&page=0&type=remux",
            None,
            {
                "page_count": 1,
                "page": [
                    _TEST_DATA_JOB_3
                ],
                "workers": [_TEST_DATA_WORKER_1]
            },
            use_moderator_login=True
        )
        self.do_json_request(
            "GET",
            "/jobs?entries_per_page=10&page=0&worker_id=none",
            None,
            {
                "page_count": 1,
                "page": [
                    _TEST_DATA_JOB_1,
                    _TEST_DATA_JOB_2,
                ],
                "workers": [_TEST_DATA_WORKER_1]
            },
            use_moderator_login=True
        )
        self.do_json_request(
            "GET",
            "/jobs?entries_per_page=10&page=0&worker_id=work1",
            None,
            {
                "page_count": 1,
                "page": [
                    _TEST_DATA_JOB_3
                ],
                "workers": [_TEST_DATA_WORKER_1]
            },
            use_moderator_login=True
        )
        self.do_json_request(
            "GET",
            "/jobs?entries_per_page=10&page=0&worker_id=work1&type=remux&state=running",
            None,
            {
                "page_count": 1,
                "page": [
                    _TEST_DATA_JOB_3
                ],
                "workers": [_TEST_DATA_WORKER_1]
            },
            use_moderator_login=True
        )
        self.do_json_request(
            "GET",
            "/jobs?entries_per_page=10&page=0&worker_id=work1&type=remux&state=finished",
            None,
            {
                "page_count": 0,
                "page": [],
                "workers": [_TEST_DATA_WORKER_1]
            },
            use_moderator_login=True
        )
    
    def test_delete_job(self):
        self.do_json_request(
            "DELETE",
            "/job/1",
            expected_response_code=HTTP_401_UNAUTHORIZED
        )
        self.do_json_request(
            "DELETE",
            "/job/1",
            use_moderator_login=True
        )
        self.do_json_request(
            "GET",
            "/jobs?entries_per_page=10&page=0&state=deleted",
            None,
            {
                "page_count": 1,
                "page": [
                    _TEST_DATA_JOB_1,
                    _TEST_DATA_JOB_2,
                ],
                "workers": [_TEST_DATA_WORKER_1]
            },
            ignore_value_keys=["$.page[0].state"],
            use_moderator_login=True
        )
    
    def test_job_action_restart(self):
        self.do_json_request(
            "POST",
            "/job/5/action",
            {
                "action": "restart"
            },
            expected_response_code=HTTP_401_UNAUTHORIZED
        )
        self.do_json_request(
            "POST",
            "/job/5/action",
            {
                "action": "restart"
            },
            use_moderator_login=True
        )
        self.do_json_request(
            "GET",
            "/jobs?entries_per_page=10&page=0&state=ready",
            None,
            {
                "page_count": 1,
                "page": [
                    _TEST_DATA_JOB_1,
                    _TEST_DATA_JOB_5,
                ],
                "workers": [_TEST_DATA_WORKER_1]
            },
            ignore_value_keys=["$.page[1].state"],
            use_moderator_login=True
        )
        
    def test_job_action_cancel(self):
        self.do_json_request(
            "POST",
            "/job/3/action",
            {
                "action": "cancel"
            },
            expected_response_code=HTTP_401_UNAUTHORIZED
        )
        self.do_json_request(
            "POST",
            "/job/3/action",
            {
                "action": "cancel"
            },
            use_moderator_login=True
        )
        self.do_json_request(
            "GET",
            "/jobs?entries_per_page=10&page=0&state=running",
            None,
            {
                "page_count": 1,
                "page": [
                    _TEST_DATA_JOB_3 | {"is_canceled": True}
                ],
                "workers": [_TEST_DATA_WORKER_1]
            },
            use_moderator_login=True
        )
        
    def test_job_action_copy(self):
        self.do_json_request(
            "POST",
            "/job/4/action",
            {
                "action": "copy"
            },
            expected_response_code=HTTP_401_UNAUTHORIZED
        )
        self.do_json_request(
            "POST",
            "/job/4/action",
            {
                "action": "copy"
            },
            use_moderator_login=True
        )
        _TEST_DATA_NEW_JOB_ = {
            "type": "thumbnail",
            "priority": 0,
            "state": "ready",
            "is_canceled": False,
            "data": "{\"src\": \"pub/07ws-diskrete/07ws-diskrete-071211.mp4\", \"filename\": \"l_25.jpg\"}",
            "status": None
        }
        self.do_json_request(
            "GET",
            "/jobs?entries_per_page=10&page=0&state=ready",
            None,
            {
                "page_count": 1,
                "page": [
                    _TEST_DATA_NEW_JOB_,
                    _TEST_DATA_JOB_1,
                ],
                "workers": [_TEST_DATA_WORKER_1]
            },
            ignore_value_keys=["$.page[0].id", "$.page[0].creation_time"],
            use_moderator_login=True
        )
