from api.miscellaneous.constants import *

from api_test import ApiTest


class UserTest(ApiTest):
    
    def assert_valid_user_info(self, info: dict):
        self.assert_contains(info, "id")
        self.assert_contains(info, "id_string")
        self.assert_contains(info, "name")
    
    def test_get_all(self):
        self.do_json_request(
            "GET",
            "/users",
            expected_response_code=HTTP_401_UNAUTHORIZED
        )
        users = self.do_json_request(
            "GET",
            "/users",
            use_moderator_login=True,
        )[1]["users"]
        for user in users:
            self.assert_valid_user_info(user)
    
    def test_get_settings(self):
        self.do_json_request(
            "GET",
            "/user/me/settings",
            expected_response_code=HTTP_401_UNAUTHORIZED
        )
        self.do_json_request(
            "GET",
            "/user/me/settings",
            use_moderator_login=True,
        )
        
    def test_update_settings(self):
        self.do_json_request(
            "PATCH",
            "/user/me/settings",
            expected_response_code=HTTP_401_UNAUTHORIZED
        )
        self.do_json_request(
            "PATCH",
            "/user/me/settings",
            {
                "updates": {
                    "definitely_non_existent_setting": True
                }
            },
            use_moderator_login=True,
            expected_response_code=HTTP_400_BAD_REQUEST
        )
        # This test might break when new settings are added
        entry = self.do_json_request(
            "GET",
            "/user/me/settings",
            use_moderator_login=True,
        )[1]["sections"][0]["entries"][0]
        self.assertEqual(entry["id"], "notifications_enabled")
        self.assertEqual(entry["value"], True)
        self.do_json_request(
            "PATCH",
            "/user/me/settings",
            {
                "updates": {
                    "notifications_enabled": False
                }
            },
            use_moderator_login=True,
        )
        self.assertEqual(self.do_json_request(
            "GET",
            "/user/me/settings",
            use_moderator_login=True,
        )[1]["sections"][0]["entries"][0]["value"], False)
